﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Windows.Forms;


namespace Mandelbrot
{
    public partial class Form1 : Form
    {
        private int _width;
        private int _height;
        private Mandelbrotova _mnozina;
        private ParMandel _pmnozina; //starší přístup 
        private ParMandel2 _pmnozina2;
        private bool _zooming;
        private List<ZoomMouseClick> _zoom;

        private int Iterace { get; set; }
        private int Cores { set; get; }
        private static bool Para { get; set; } = true;

        public Form1()
        {
            InitializeComponent();
            _width = mandelbrot_img.Width;
            _height = mandelbrot_img.Height;
            lblwidth.Text = _width.ToString();
            lblheight.Text = _height.ToString();
            for (int i = 1; i <= Environment.ProcessorCount; i++)
            {
                cores_select.Items.Add(i);
            }
            cores_select.SelectedIndex = 0;
            _zoom = new List<ZoomMouseClick>();
        }

        private void Form1_SizeChanged(object sender, EventArgs e) /*Změní-li se velikost okýnka změníme velikost obrázku*/
        {
            mandelbrot_img.Size = Size;
            _width = mandelbrot_img.Width;
            _height = mandelbrot_img.Height;
            lblwidth.Text = _width.ToString();
            lblheight.Text = _height.ToString();
        }

        private void btnStart_Click(object sender, EventArgs e)/*nová množina*/
        {            
            Iterace = !int.TryParse(txtIterace.Text, out var iterace) ? 1000 : iterace;

            if (mandelbrot_img.Image == null) //obrázek neexistuje můeme kreslit
            {
                if (Para)
                    Paralelne();
                else
                    Sekvencne();
            }
            else
            {
                mandelbrot_img.Image.Dispose();
                if (Para)
                    Paralelne();
                else
                    Sekvencne();
            }
        }

        private void Paralelne()
        {
            //_pmnozina = new ParMandel(mandelbrot_img.Width, mandelbrot_img.Height, 2, -2, 2, -2, Iterace, Cores);
            //Stopwatch t = new Stopwatch();
            //t.Start();
            //mandelbrot_img.Image = _pmnozina.Mandel();
            //t.Stop();
            //cas.Text = t.Elapsed.ToString();


            Stopwatch t = new Stopwatch();
            t.Start();
            _pmnozina2 = new ParMandel2(_width, _height, 2, -2, 2, -2, Iterace, Cores);
            mandelbrot_img.Image = _pmnozina2.Mandel();
            t.Stop();
            cas.Text = t.Elapsed.ToString();
        }

        private void Sekvencne()
        {
            _mnozina = new Mandelbrotova(_width, _height, 2, -2, 2, -2, Iterace);
            Stopwatch t = new Stopwatch();
            t.Start();
            mandelbrot_img.Image = _mnozina.Mandel();
            t.Stop();
            cas.Text = t.Elapsed.ToString();
        }

        private void rBPara_CheckedChanged(object sender, EventArgs e)
        {
            Para = rBPara.Checked;
        }

        private void Zoom(object sender, MouseEventArgs e)
        {  
            if (!_zooming)
            {
                if(e.Delta != 1000)
                {
                    _zoom.Add(new ZoomMouseClick(e.X,e.Y));
                    ZoomLevel.Text = _zoom.Count.ToString();
                }
                Stopwatch t = new Stopwatch();
                _zooming = true;
                if (mandelbrot_img.Image != null) //nemůžeme zoomovat prázdný obrázek
                {
                    mandelbrot_img.Image.Dispose();
                    Task.Factory.StartNew(() =>
                    {
                        t.Start();
                        mandelbrot_img.Image = Para ? _pmnozina2.Zoom(e.X, e.Y) : _mnozina.Zoom(e.X, e.Y);
                    }).Wait();
                }
                t.Stop();
                cas.Text = t.Elapsed.ToString();
                _zooming = false;
            }
        }

        private void cores_select_SelectedIndexChanged(object sender, EventArgs e)
        {
            Cores = int.Parse(cores_select.SelectedItem.ToString());
        }

        private void threads_Leave(object sender, EventArgs e)
        {
            if (threads.Text != "")
            {
                try
                {
                    Cores = int.Parse(threads.Text);
                }
                catch (FormatException)
                {
                    MessageBox.Show(@"Pole počet vláken může obsahovat pouze celá čísla.");
                    threads.Text = "";
                }
                catch (OverflowException)
                {
                    MessageBox.Show(@"Zadaný číslo je příliš velký.");
                    threads.Text = "";
                }
            }
        }

        private void btnTest_Click(object sender, EventArgs e)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();
            Task.Factory.StartNew(()=>btnStart_Click(sender, e)).Wait();

            foreach (ZoomMouseClick zoom in _zoom)
            {
                Task.Factory.StartNew(() =>
                {
                    MouseEventArgs m = new MouseEventArgs(MouseButtons.Left, 1, zoom.X, zoom.Y, 1000);
                    Zoom(sender, m);
                }).Wait();
            }
            stopwatch.Stop();
            MessageBox.Show(stopwatch.Elapsed.ToString());
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            _zoom = new List<ZoomMouseClick>();
            ZoomLevel.Text = "0";
        }
    }

}