﻿using System;
using System.Drawing;
using System.Threading.Tasks;

namespace Mandelbrot
{
    public class ParMandel : Mandelbrotova
    {
        private readonly int _tasks;
        public ParMandel(int width, int height, double maxr, double minr, double maxi, double mini, int iterationCount, int tasks) : base(width, height, maxr, minr, maxi, mini, iterationCount)
        {
            _tasks = tasks;
        }
        public override Bitmap Mandel() 
        {
            Bitmap img = new Bitmap(Width, Height);
            object zamek = new object();
            Task[] ukoly = new Task[_tasks];
            int[,] pole = new int[Width, Height];
            for (int i = 0; i < _tasks; i++)
            {
                int i1 = i;
                ukoly[i] = new Task(() =>
                {
                    double dx;
                    double dy;
                    double cx;
                    double cy;
                    double posunX = ((Maxr - Minr) / Width);
                    double posunY = ((Maxi - Mini) / Height);
                    for (int x = (i1 * Width / _tasks); x < (Width / _tasks * (i1 + 1)); x++)
                    {
                        cx = (posunX * x) - Math.Abs(Minr);
                        for (int y = 0; y < Height; y++)
                        {
                            dx = 0;
                            dy = 0;
                            cy = (posunY * y) - Math.Abs(Mini);
                            pole[x, y] = Pixel(dx, dy, cx, cy);
                            // špatná myšlenka - zámek algoritmus šíleně zpomalí
                            //lock (zamek)
                            //{ 
                            //    int loopgo = Pixel(dx, dy, cx, cy);
                            //    Color c = Color.FromArgb(loopgo % 128 * 2,
                            //        loopgo % 32 * 7, loopgo % 16 * 14);
                            //    img.SetPixel(x, y, c);
                            //}
                        }
                    }
                });
                ukoly[i].Start();
            }
            Task.WaitAll(ukoly);
            for (int i = 0; i < Width; i++)
            {
                for (int j = 0; j < Height; j++)
                {
                    img.SetPixel(i, j, Color.FromArgb(pole[i, j] % 128 * 2, pole[i, j] % 32 * 7, pole[i, j] % 16 * 14));
                }
            }
            return img;
        }
        private int Pixel(double dx, double dy, double cx, double cy) /*cyklus který rozhodne jestli bod do množiny patří či ne*/
        {
            double tempdx;
            int count = 0;

            while (dx * dx + dy * dy <= 4 && count < IterationCount)
            {
                count++;
                tempdx = dx;
                dx = dx * dx - dy * dy + cx;
                dy = 2 * tempdx * dy + cy;
            }
            return count != IterationCount ? count : 0;
        }
    }
}