﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mandelbrot
{
    public class ParMandel2
    {
        private double Maxr { get; set; }
        private double Minr { get; set; }
        private double Maxi { get; set; }
        private double Mini { get; set; }
        private int Width { get; set; }
        private int Height { get; set; }
        private Bitmap _finalBitmap;
        private List<Bitmap> _images;
        private int Threads { get; set; }


        public ParMandel2(int width, int height, double maxr, double minr, double maxi, double mini, int threads)
        {
            Maxr = maxr;
            Minr = minr;
            Maxi = maxi;
            Mini = mini;
            Width = width;
            Height = height;
            _finalBitmap = new Bitmap(Width, Height);
            _images = new List<Bitmap>();
            Threads = threads;

        }

        public Bitmap Pmandel()
        {
            if (_images.Count > 0) //uz jsme kreslili? je potřeba nová bitmapa
            {
                _finalBitmap.Dispose();
                _finalBitmap = new Bitmap(Width, Height);
                _images.Clear();
            }
            Task[] ukoly = new Task[Threads];
            int[,] pole = new int[Width, Height];
            /*Logika na více procesoru*/
            int[] meze = Meze(Width, Threads);
            for (int i = 0; i < Threads; i++)
            {
                int m = i;
                ukoly[i] = new Task(() =>
                {
                    double dx;
                    double dy;
                    double cx;
                    double cy;
                    int color;
                    Bitmap obr = new Bitmap(Width, Height);
                    double posunX = ((Maxr - Minr) / Width);
                    double posunY = ((Maxi - Mini) / Height);
                    for (int x = meze[m]; x < meze[m + 1]; x++)
                    {
                        cx = (posunX * x) - Math.Abs(Minr);
                        for (int y = 0; y < Height; y++)
                        {
                            dx = 0;
                            dy = 0;
                            cy = (posunY * y) - Math.Abs(Mini);
                            color = Pixel(dx, dy, cx, cy);
                            obr.SetPixel(x, y, Color.FromArgb(color % 128 * 2, color % 32 * 7, color % 16 * 14));
                        }
                    }
                    _images.Add(obr);
                });
                ukoly[i].Start();
            }
            /*************************/
            Task.WaitAll(ukoly);

            Graphics g = Graphics.FromImage(_finalBitmap);

            //pozadí
            //g.Clear(Color.Blue);

            foreach (Bitmap image in _images)
            {
                g.DrawImage(image, new Rectangle(0, 0, image.Width, image.Height)); //0 ,0 protože předpokládám že pixely jsou na bimapách přesně na místech
            }

            return _finalBitmap;
        }

        private int Pixel(double dx, double dy, double cx, double cy) /*funkce který rozhodne jestli bod do množiny patří či ne*/
        {
            double tempdx;
            const int maxPocetiteraci = 1000;
            int pocetiteraci = 0;

            while (dx * dx + dy * dy <= 4 && pocetiteraci < maxPocetiteraci)
            {
                pocetiteraci++;
                tempdx = dx;
                dx = (dx * dx) - (dy * dy) + cx;
                dy = (2 * tempdx * dy) + cy;
            }
            return pocetiteraci != maxPocetiteraci ? pocetiteraci : 0;
        }

        public Bitmap Zoom(int mouseX, int mouseY)
        {
            double posunX = ((Maxr - Minr) / Width);
            double posunY = ((Maxi - Mini) / Height);

            int zoomx = Width / 5;
            int zoomy = Height / 5;
            Maxr = (mouseX + zoomx) * posunX - Math.Abs(Minr);
            Minr = (mouseX - zoomx) * posunX - Math.Abs(Minr);
            Maxi = (mouseY + zoomy) * posunY - Math.Abs(Mini);
            Mini = (mouseY - zoomy) * posunY - Math.Abs(Mini);
            Bitmap img = Pmandel();
            return img;
        }

        private int[] Meze(int width, int threads)
        {/*funkce vypočte meze pro každý kus obrázku*/
            /*pokud se to dělí přímo v cyklu tak vznikají bílé pruhy (díry) v bitmapě*/
            int w = width / threads;
            int[] meze = new int[threads + 1]; //chci přidat ještě nulu
            meze[0] = 0;
            for (int x = 1; x <= threads; x++)
            {
                meze[x] = x * w;
                if (meze[x] + w >= width)
                {
                    meze[x] = meze[x] + (width - meze[x]);
                }
            }
            return meze;
        }
    }
}