﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Threading;
using System.Windows.Forms;

namespace Paralelni_nasobeni_matic
{
    public partial class Form1 : Form
    {
        private int[,] _matA;
        private int[,] _matB;
        private int _rank;
        private Stopwatch _timer;
        private Action _act;
        private ParalelMatrixClass _paralel;
        private SerialMatrixClass _serial;
        private CancellationTokenSource _tokenSource;
        public Form1()
        {
            InitializeComponent();
            _timer = new Stopwatch();
            InitializeMatrixes();
        }

        private void btn_para_Click(object sender, EventArgs e)
        {
            _act = Paralel;
            backgroundWorker1.RunWorkerAsync();        
        }

        private void btn_ser_Click(object sender, EventArgs e)
        {
            _act = Serial;
            backgroundWorker1.RunWorkerAsync();
        }

        private void btn_generate_Click(object sender, EventArgs e)
        {
            if (!int.TryParse(txt_rank.Text, out _rank)) return;
            InitializeMatrixes();
            Generator(_rank, ref _matA);
            Generator(_rank, ref _matB);
        }

        private void InitializeMatrixes()
        {
            if (_rank < 0) return;
            _matA = new int[_rank, _rank];
            _matB = new int[_rank,_rank];
        }

        private void Generator(int rank, ref int[,] mat)
        {
            var rnd = new Random();

            for (int j = 0; j < rank; j++)
            {
                for (int i = 0; i < rank; i++)
                {
                    mat[j, i] = rnd.Next(10);
                }
            }
        }

        private void Paralel()
        {
            _paralel = new ParalelMatrixClass(_matA, _matB);
            _paralel.Multiply(_tokenSource.Token);
        }

        private void Serial()
        {
            _serial = new SerialMatrixClass(_matA, _matB);
            _serial.Multiply(_tokenSource.Token);         
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            lblElapsed.Text = _timer.Elapsed.ToString();
            //lst_vysledek.DataSource = _paralel == null ? _serial.ResultAsStringList() : _paralel.ResultAsStringList();
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            _tokenSource= new CancellationTokenSource();
            _timer.Reset();
            _timer.Start();
            if (chkTest.Checked)
            {
                TestClass testClass = new TestClass();
                testClass.TestFuntion(_act);
                testClass.TimeResultsToFile(_act.Method.Name);
            }
            else
            {
                _act();
            }
            _timer.Stop();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            backgroundWorker1.CancelAsync();
            _tokenSource.Cancel();
        }
    }
}